<?php
/**
 * Anwiki is a multilingual content management system <http://www.anwiki.com>
 * Copyright (C) 2007-2009 Antoine Walter <http://www.anw.fr>
 * 
 * Anwiki is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 * 
 * Anwiki is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Anwiki.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Importing contents into Anwiki from an XML file.
 * @package Anwiki
 * @version $Id: action_import.php 170 2009-04-05 18:06:49Z anw $
 * @copyright 2007-2009 Antoine Walter
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License 3
 */
 
class AnwActionDefault_import extends AnwActionGlobal
{
	const XMLTAG_PAGEGROUP = "anwpagegroup";
	const XMLTAG_PAGE = "anwpage";
	
	//inputs
	const IN_UPLOAD = "uploadfile";
	const IN_FILENAME = "filename";
	const IN_PREFIX = "importprefix";
	const IN_PAGENAMES = "importpages";
	
	const CSS_FILENAME = "action_import.css";
	
	
	function getNavEntry()
	{
		return $this->createManagementGlobalNavEntry();
	}
	
	function run()
	{
		// load CSS
		$this->head( $this->getCssSrcComponent(self::CSS_FILENAME) );
		
		$this->setTitle( $this->t_("title") );
		
		if (AnwEnv::_POST(self::IN_FILENAME) && AnwEnv::_POST(self::IN_PAGENAMES))
		{
			//4) do import
			$this->doImport(AnwEnv::_POST(self::IN_FILENAME), AnwEnv::_POST(self::IN_PAGENAMES));
		}
		else if(AnwEnv::_GET(self::IN_FILENAME))
		{
			//3) form: select pages to import
			$this->importSelectionForm(AnwEnv::_GET(self::IN_FILENAME), AnwEnv::_GET(self::IN_PREFIX, ""));
		}
		else if (AnwEnv::_FILES(self::IN_UPLOAD))
		{
			//2) do upload of an export file
			$this->doUpload(AnwEnv::_FILES(self::IN_UPLOAD), AnwEnv::_POST(self::IN_PREFIX, ""));
		}
		else
		{
			//1) form: upload an export file
			$this->uploadForm();
		}
	}
	
	private function uploadForm()
	{
		$this->out .= $this->tpl()->uploadForm($this->linkMe(), self::IN_UPLOAD, self::IN_PREFIX);
	}
	
	private function doUpload($mUploadedFile, $sPrefix)
	{
		//move uploaded file
		$sDestinationFileName = AnwUtils::genStrongRandMd5();
		AnwUtils::move_uploaded_file($mUploadedFile, self::tmpFilename($sDestinationFileName));
		
		//redirect
		$sUrl = AnwUtils::aLink($this->getName(), 
							array(self::IN_FILENAME=>$sDestinationFileName,
							self::IN_PREFIX=>$sPrefix));
		AnwUtils::redirect($sUrl);
	}
	
	private function importSelectionForm($sUploadedFile, $sImportPrefix)
	{
		//load XML from file
		$aaData = $this->getDataFromXmlFile(self::tmpFilename($sUploadedFile));
		
		$sExportTime = Anwi18n::dateTime($aaData['TIME']);
		$sExportFrom = $aaData['FROM'];
		$sExportVersionName = $aaData['VERSION_NAME'];
		$nExportVersionId = $aaData['VERSION_ID'];
		
		//select pages to import
		$this->out .= $this->tpl()->beginSelection($this->linkMe(), $sUploadedFile, self::IN_FILENAME, $sExportTime, $sExportFrom, $sExportVersionName);
		
		foreach ($aaData['PAGEGROUPS'] as $aaDataGroup)
		{
			$this->out .= $this->tpl()->rowGroupOpen($sExportTime, $sExportFrom);
			
			foreach ($aaDataGroup['PAGES'] as $aaDataPage)
			{
				$asNotices = array();
				$bImportDisabled = false;
				
				$sOriginalPageName = $aaDataPage['NAME'];
				$sPageName = $sImportPrefix.$sOriginalPageName; //prefix
				$sPageLang = $aaDataPage['LANG'];
				$nPageTime = $aaDataPage['TIME'];
				$sPageContent = $aaDataPage['CONTENT'];
				
				try
				{
					//check pagename and content
					$asNotices = $this->checkPermissions($sPageName, $sPageLang, $sPageContent);
					
					foreach ($asNotices as $sNotice)
					{
						//don't disable import checkbox for pages already existing, as user may change the destination name
						if ($sNotice != $this->t_("notice_exists"))
						{
							$bImportDisabled = true;
						}
					}
				}
				catch(AnwInvalidContentException $e)
				{
					$asNotices[] = $this->t_("notice_content");
					$bImportDisabled = true;
				}
				catch(AnwException $e)
				{
					$asNotices[] = $this->t_("notice_unknown");
					$bImportDisabled = true;
				}
				
				//check lang
				if (!Anwi18n::langExists($sPageLang))
				{
					$asNotices[] = $this->t_("notice_lang");
					$bImportDisabled = true;
				}
				
				$this->out .= $this->tpl()->rowTranslation(self::IN_PAGENAMES, $sOriginalPageName, $sPageName, $sPageLang, $nPageTime, $bImportDisabled, $asNotices);
			}
			
			$this->out .= $this->tpl()->rowGroupClose();
		}
		
		$this->out .= $this->tpl()->endSelection();
	}
	
	private function doImport($sUploadedFile, $asSelectedPages)
	{
		//import selected files
		$this->out .= $this->tpl()->beginProcess();
		
		//load XML from file
		$aaData = $this->getDataFromXmlFile(self::tmpFilename($sUploadedFile));
		
		//delete tmp file
		AnwUtils::unlink(self::tmpFilename($sUploadedFile), ANWPATH_TMP);
		
		AnwStorage::transactionStart();
		try
		{
			foreach ($aaData['PAGEGROUPS'] as $aaDataGroup)
			{
				$this->out .= $this->tpl()->rowGroupOpen();
				
				$oFirstPage = null;
				$oContentClass = AnwContentClasses::getContentClass($aaDataGroup['CONTENTCLASS']);
	
				foreach ($aaDataGroup['PAGES'] as $aaDataPage)
				{
					$asNotices = array();
					
					$sOriginalPageName = $aaDataPage['NAME'];
					
					$sPageName = AnwEnv::_POST("importpages_".$sOriginalPageName."_name");
					$sPageLang = AnwEnv::_POST("importpages_".$sOriginalPageName."_lang");				
					
					$sPageContent = $aaDataPage['CONTENT'];
					
					
					//do we want to import this page?
					if (in_array($sOriginalPageName, $asSelectedPages) && $sPageName && $sPageLang)
					{
						//try
						//{
						//check pagename and content
						
						$asNotices = $this->checkPermissions($sPageName, $sPageLang, $sPageContent);
						
						if (count($asNotices) == 0)
						{
							$oContent = $oContentClass->rebuildContentFromXml($sPageContent);
							
							if (!$oFirstPage)
							{
								$oPage = AnwPage::createNewPage($oContentClass, $sPageName, $sPageLang, "", $oContent);
								$oFirstPage = $oPage;
							}
							else
							{
								$oPage = $oFirstPage->createNewTranslation($sPageName, $sPageLang, "", $oContent);
							}
							
							$this->out .= $this->tpl()->rowTranslationProcess_success($oPage->link());																		
						}
						else
						{
							$this->out .= $this->tpl()->rowTranslationProcess_failed($sPageName, $sPageLang, $asNotices);
						}
						//}
						//catch(AnwException $e)
						//{
						//	$asNotices[] = $this->t_("notice_unknown");print $e->getFile().'!'.$e->getLine();print_r($e->getTrace());
						//}
						
						
					}
					else
					{
						$this->out .= $this->tpl()->rowTranslationProcess_skipped($sPageName, $sPageLang);
					}					
				}
				$this->out .= $this->tpl()->rowGroupClose();
			}
			AnwStorage::transactionCommit();	
		}
		catch(AnwException $e)
		{
			AnwStorage::transactionRollback();
			throw $e;
		}
		$this->out .= $this->tpl()->endProcess();
	}
	
	
	//----------------
	
	
	private function checkPermissions($sPageName, $sPageLang, $sPageContent)
	{
		$asNotices = array();
		
		//check that page don't exist
		if (!AnwPage::isAvailablePageName($sPageName))
		{
			$asNotices[] = $this->t_("notice_exists");
		}
		
		//check PHP permission
		if (AnwUtils::contentHasPhpCode($sPageContent) && !AnwCurrentSession::getUser()->isPhpEditionAllowed())
		{
			$asNotices[] = $this->t_("notice_php");
		}
		
		//check JS permission
		if (AnwUtils::contentHasJsCode($sPageContent) && !AnwCurrentSession::getUser()->isJsEditionAllowed())
		{
			$asNotices[] = $this->t_("notice_js");
		}
		
		//check ACL permission : create and edit
		if (!AnwCurrentSession::isActionAllowed($sPageName, "create", $sPageLang) 
			||!AnwCurrentSession::isActionAllowed($sPageName, "edit", $sPageLang))
		{
			$asNotices[] = $this->t_("notice_acl");
		}
		
		return $asNotices;
	}
	
	
	private static function tmpFilename($sFilename)
	{
		return ANWPATH_TMP.$sFilename.'.xml';
	}
	
	
	private function getDataFromXmlFile($sFilename)
	{
		//load XML from file
		if (!file_exists($sFilename)) throw new AnwBadCallException("import file not found");
		$sFileContent = AnwUtils::file_get_contents($sFilename);
		$oRootNode = AnwUtils::loadXml($sFileContent);
				
		$aaData = array();	
		$aaData['PAGEGROUPS'] = array();
		$aaData['TIME'] = $oRootNode->getAttribute("time");
		$aaData['FROM'] = $oRootNode->getAttribute("from");
		$aaData['VERSION_ID'] = $oRootNode->getAttribute("version_id");
		$aaData['VERSION_NAME'] = $oRootNode->getAttribute("version_name");
		
		$aoPageGroupsNodes = AnwXml::xmlGetChildsByTagName(self::XMLTAG_PAGEGROUP, $oRootNode);
		foreach ($aoPageGroupsNodes as $oPageGroupNode)
		{
			$aaDataGroup = array();
			$aaDataGroup['CONTENTCLASS'] = $oPageGroupNode->getAttribute("contentclass");
			$aaDataGroup['PAGES'] = array();
			
			$aoTranslationsNodes = AnwXml::xmlGetChildsByTagName(self::XMLTAG_PAGE, $oPageGroupNode);
			foreach ($aoTranslationsNodes as $oPageNode)
			{
				$asDataPage = array();
				$asDataPage['NAME'] = $oPageNode->getAttribute("name");
				$asDataPage['LANG'] = $oPageNode->getAttribute("lang");
				$asDataPage['TIME'] = (int)$oPageNode->getAttribute("time");
				$asDataPage['CONTENT'] = AnwUtils::xmlDumpNodeChilds($oPageNode);
				
				$aaDataGroup['PAGES'][] = $asDataPage;
			}			
			$aaData['PAGEGROUPS'][] = $aaDataGroup;
		}
		return $aaData;
	}
}

?>